#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   settings.py
@Time    :   2020/08/15
@Author  :   Yaronzz
@Version :   1.0
@Contact :   yaronhuang@foxmail.com
@Desc    :   
'''
import json
import base64
from aigpy.fileHelper import getFileContent, write
from aigpy.modelHelper import dictToModel, modelToDict
from tidal_dl.enum import AudioQuality, VideoQuality



def __encode__(string):
    sw = bytes(string, 'utf-8')
    st = base64.b64encode(sw)
    return st

def __decode__(string):
    try:
        sr = base64.b64decode(string)
        st = sr.decode()
        return st
    except:
        return string
    

class UserSettings(object):
    userid = None
    username = None
    password = None
    sessionid1 = None
    sessionid2 = None
    countryCode = None
    assesstoken = None

    @staticmethod
    def read():
        txt = getFileContent('./usersettings.json', True)
        if txt == "":
            return UserSettings()
        txt = __decode__(txt)
        data = json.loads(txt)
        ret = dictToModel(data, UserSettings())
        return ret
    
    @staticmethod
    def save(model):
        data = modelToDict(model)
        txt = json.dumps(data)
        txt = __encode__(txt)
        write('./usersettings.json', txt, 'wb')

class Settings(object):
    downloadPath = "./download/"
    onlyM4a = False
    addExplicitTag = True
    addHyphen = True
    addYear = False
    useTrackNumber = True
    audioQuality = AudioQuality.Normal
    videoQuality = VideoQuality.P360
    checkExist = True
    artistBeforeTitle = False
    includeEP = True
    addAlbumIDBeforeFolder = False
    saveCovers = True

    @staticmethod
    def read():
        txt = getFileContent('./settings.json')
        if txt == "":
            return Settings()
        data = json.loads(txt)
        ret = dictToModel(data, Settings())
        ret.audioQuality = Settings.getAudioQuality(ret.audioQuality)
        ret.videoQuality = Settings.getVideoQuality(ret.videoQuality)
        return ret

    @staticmethod
    def save(model):
        data = modelToDict(model)
        data['audioQuality'] = model.audioQuality.name
        data['videoQuality'] = model.videoQuality.name
        txt = json.dumps(data)
        write('./settings.json', txt, 'w+')

    @staticmethod
    def getAudioQuality(value):
        for item in AudioQuality:
            if item.name == value:
                return item
        return AudioQuality.Normal

    @staticmethod
    def getVideoQuality(value):
        for item in VideoQuality:
            if item.name == value:
                return item
        return VideoQuality.P360




#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   printf.py
@Time    :   2020/08/16
@Author  :   Yaronzz
@Version :   1.0
@Contact :   yaronhuang@foxmail.com
@Desc    :   
'''
import prettytable
from aigpy.cmdHelper import red, green, blue, yellow, myprint, myinput, TextColor

__LOGO__ = '''
 /$$$$$$$$ /$$       /$$           /$$               /$$ /$$
|__  $$__/|__/      | $$          | $$              | $$| $$
   | $$    /$$  /$$$$$$$  /$$$$$$ | $$          /$$$$$$$| $$
   | $$   | $$ /$$__  $$ |____  $$| $$ /$$$$$$ /$$__  $$| $$
   | $$   | $$| $$  | $$  /$$$$$$$| $$|______/| $$  | $$| $$
   | $$   | $$| $$  | $$ /$$__  $$| $$        | $$  | $$| $$
   | $$   | $$|  $$$$$$$|  $$$$$$$| $$        |  $$$$$$$| $$
   |__/   |__/ \_______/ \_______/|__/         \_______/|__/
   
       https://github.com/yaronzz/Tidal-Media-Downloader 
'''
VERSION = '2020.8.19.4'

class Printf(object):

    @staticmethod
    def logo():
        print(__LOGO__)
        print('                      v' + VERSION)

    @staticmethod
    def settings(data):
        tb = prettytable.PrettyTable()
        tb.field_names = [green("SETTINGS"), green("VALUE")]
        tb.align = 'l'
        tb.add_row(["Download path", data.downloadPath])
        tb.add_row(["Convert mp4 to m4a", data.onlyM4a])
        tb.add_row(["Add explicit tag", data.addExplicitTag])
        tb.add_row(["Add hyphen", data.addHyphen])
        tb.add_row(["Add user track number", data.useTrackNumber])
        tb.add_row(["Audio quality", data.audioQuality])
        tb.add_row(["Video quality", data.videoQuality])
        tb.add_row(["Check exist", data.checkExist])
        tb.add_row(["ArtistName before track-title", data.artistBeforeTitle])
        tb.add_row(["Id Before album-folder", data.addAlbumIDBeforeFolder])
        tb.add_row(["Include single&ep", data.includeEP])
        tb.add_row(["Save covers", data.saveCovers])
        print(tb)

    @staticmethod
    def choices():
        print("=====================Choice=========================")
        tb = prettytable.PrettyTable()
        tb.field_names = ["CHOICES", "FUNCTION"]
        tb.align = 'l'
        tb.set_style(prettytable.PLAIN_COLUMNS)
        tb.add_row([green("Enter '0':"), "Exit."])
        tb.add_row([green("Enter '1':"), "Login."])
        tb.add_row([green("Enter '2':"), "Settings."])
        tb.add_row([green("Enter '3':"), "Set AccessToken."])
        tb.add_row([green("Enter 'Url/ID':"), "Download by url or id."])
        print(tb)
        print("====================================================")

    @staticmethod
    def enter(string):
        myprint(string, TextColor.Yellow, None)
        ret = myinput("")
        return ret

    @staticmethod
    def err(string):
        print(red("[ERR] ") + string)
    
    @staticmethod
    def info(string):
        print(blue("[INFO] ") + string)

    @staticmethod
    def success(string):
        print(green("[SUCCESS] ") + string)

    @staticmethod
    def album(data):
        tb = prettytable.PrettyTable()
        tb.field_names = [green("ALBUM-PROPERTY"), green("VALUE")]
        tb.align = 'l'
        tb.add_row(["Title", data.title])
        tb.add_row(["Track Number", data.numberOfTracks])
        tb.add_row(["Video Number", data.numberOfVideos])
        tb.add_row(["Release Date", data.releaseDate])
        tb.add_row(["Version", data.version])
        tb.add_row(["Explicit", data.explicit])
        print(tb)

    @staticmethod
    def track(data):
        tb = prettytable.PrettyTable()
        tb.field_names = [green("TRACK-PROPERTY"), green("VALUE")]
        tb.align = 'l'
        tb.add_row(["Title", data.title])
        tb.add_row(["Album", data.album.title])
        tb.add_row(["Version", data.version])
        tb.add_row(["Explicit", data.explicit])
        print(tb)
    
    @staticmethod
    def video(data):
        tb = prettytable.PrettyTable()
        tb.field_names = [green("VIDEO-PROPERTY"), green("VALUE")]
        tb.align = 'l'
        tb.add_row(["Title", data.title])
        tb.add_row(["Album", data.album.title if data.album != None else None])
        tb.add_row(["Version", data.version])
        tb.add_row(["Explicit", data.explicit])
        print(tb)

    @staticmethod
    def artist(data):
        tb = prettytable.PrettyTable()
        tb.field_names = [green("ARTIST-PROPERTY"), green("VALUE")]
        tb.align = 'l'
        tb.add_row(["ID", data.id])
        tb.add_row(["Name", data.name])
        tb.add_row(["Type", str(data.type)])
        print(tb)

    @staticmethod
    def playlist(data):
        tb = prettytable.PrettyTable()
        tb.field_names = [green("PLAYLIST-PROPERTY"), green("VALUE")]
        tb.align = 'l'
        tb.add_row(["Title", data.title])
        tb.add_row(["Track Number", data.numberOfTracks])
        tb.add_row(["Video Number", data.numberOfVideos])
        print(tb)




#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   enum.py
@Time    :   2020/08/08
@Author  :   Yaronzz
@Version :   2.0
@Contact :   yaronhuang@foxmail.com
@Desc    :   
'''
from enum import Enum

class AudioQuality(Enum):
    Normal = 0
    High = 1
    HiFi = 2
    Master = 3

class VideoQuality(Enum):
    P240 = 240
    P360 = 360
    P480 = 480
    P720 = 720
    P1080 = 1080

class Type(Enum):
    Album = 0
    Artist = 1
    Playlist = 2
    Track = 3
    Video = 4
    Null = 5



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   __init__.py
@Time    :   2020/08/15
@Author  :   Yaronzz
@Version :   1.0
@Contact :   yaronhuang@foxmail.com
@Desc    :   
'''
import os
import requests
import prettytable
from aigpy.stringHelper import isNull
from aigpy.pathHelper import mkdirs
from aigpy.pipHelper import getLastVersion
from aigpy.versionHelper import cmpVersion
from tidal_dl.tidal import TidalAPI
from tidal_dl.settings import Settings, UserSettings
from tidal_dl.printf import Printf, VERSION
from tidal_dl.download import start
from tidal_dl.enum import AudioQuality, VideoQuality

API = TidalAPI()
USER = UserSettings.read()
CONF = Settings.read()
TOKEN1,TOKEN2 = API.getToken()



def login(username="", password=""):
    while True:
        if isNull(username) or isNull(password):
            print("----------------LogIn------------------")
            username = Printf.enter("username:")
            password = Printf.enter("password:")
        msg, check = API.login(username, password, TOKEN1)
        if check == False:
            Printf.err(msg)
            username = ""
            password = ""
            continue
        api2 = TidalAPI()
        msg, check = api2.login(username, password, TOKEN2)
        break
    
    USER.username = username
    USER.password = password
    USER.userid = API.key.userId
    USER.countryCode = API.key.countryCode
    USER.sessionid1 = API.key.sessionId
    USER.sessionid2 = api2.key.sessionId
    UserSettings.save(USER)



def setAccessToken():
    while True:
        print("-------------AccessToken---------------")
        token = Printf.enter("accessToken('0' go back):")
        if token == '0':
            return
        msg, check = API.loginByAccessToken(token)
        if check == False:
            Printf.err(msg)
            continue
        if USER.userid != API.key.userId:
            Printf.err("User mismatch! Please use your own accesstoken.")
            continue
        break

    USER.assesstoken = token
    UserSettings.save(USER)



def checkLogin():
    if not isNull(USER.assesstoken):
        mag, check = API.loginByAccessToken(USER.assesstoken)
        if check == False:
            Printf.err("Invaild AccessToken!Please reset.")
    if not isNull(USER.sessionid1) and not API.isValidSessionID(USER.userid, USER.sessionid1):
        USER.sessionid1 = ""
    if not isNull(USER.sessionid2) and API.isValidSessionID(USER.userid, USER.sessionid2):
        USER.sessionid2 = ""
    if isNull(USER.sessionid1) or isNull(USER.sessionid2):
        login(USER.username, USER.password)



def changeSettings():
    Printf.settings(CONF)
    while True:
        choice = Printf.enter("Download path('0' not modify):")
        if choice == '0':
            choice = CONF.downloadPath
        elif not os.path.isdir(choice):
            if not mkdirs(choice):
                Printf.err('Path is error!')
                continue
        CONF.downloadPath = choice
        break
    while True:
        choice = Printf.enter("Audio quailty('0'-Normal,'1'-High,'2'-HiFi,'3'-Master):")
        if choice != '1' and choice != '2' and choice != '3' and choice != '0':
            Printf.err('Input error!')
            continue
        if choice == '0':
            CONF.audioQuality = AudioQuality.Normal
        if choice == '1':
            CONF.audioQuality = AudioQuality.High
        if choice == '2':
            CONF.audioQuality = AudioQuality.HiFi
        if choice == '3':
            CONF.audioQuality = AudioQuality.Master
        break
    while True:
        choice = Printf.enter("Video quailty('0'-1080,'1'-720,'2'-480,'3'-360):")
        if choice != '1' and choice != '2' and choice != '3' and choice != '0':
            Printf.err('Input error!')
            continue
        if choice == '0':
            CONF.videoQuality = VideoQuality.P1080
        if choice == '1':
            CONF.videoQuality = VideoQuality.P720
        if choice == '2':
            CONF.videoQuality = VideoQuality.P480
        if choice == '3':
            CONF.videoQuality = VideoQuality.P360
        break
    CONF.onlyM4a = Printf.enter("Convert mp4 to m4a('0'-No,'1'-Yes):") == '1'
    CONF.addExplicitTag = Printf.enter("Add explicit tag to file names('0'-No,'1'-Yes):") == '1'
    CONF.addHyphen = Printf.enter("Use hyphens instead of spaces in file names('0'-No,'1'-Yes):") == '1'
    CONF.addYear = Printf.enter("Add year to album folder names('0'-No,'1'-Yes):") == '1'
    CONF.useTrackNumber = Printf.enter("Add track number before file names('0'-No,'1'-Yes):") == '1'
    CONF.checkExist = Printf.enter("Check exist file befor download track('0'-No,'1'-Yes):") == '1'
    CONF.artistBeforeTitle = Printf.enter("Add artistName before track title('0'-No,'1'-Yes):") == '1'
    CONF.includeEP = Printf.enter("Include singles and EPs when downloading an artist's albums('0'-No,'1'-Yes):") == '1'
    CONF.addAlbumIDBeforeFolder = Printf.enter("Add id before album folder('0'-No,'1'-Yes):") == '1'
    CONF.saveCovers = Printf.enter("Save covers('0'-No,'1'-Yes):") == '1'
    Settings.save(CONF)


def main():
    Printf.logo()
    Printf.settings(CONF)
    checkLogin()

    onlineVer = getLastVersion('tidal-dl')
    if not isNull(onlineVer):
        icmp = cmpVersion(onlineVer, VERSION)
        if icmp > 0:
            Printf.info('Latest version: ' + onlineVer)

    while True:
        Printf.choices()
        choice = Printf.enter("Enter Choice:")
        if choice == "0":
            return
        elif choice == "1":
            login()
        elif choice == "2":
            changeSettings()
        elif choice == "3":
            setAccessToken()
        else:
            start(USER, CONF, choice)

if __name__ == "__main__":
    main()



